import os
import logging
import shutil
from pathlib import Path

from tinytag import TinyTag
from pydantic import BaseModel

from jgutils.filesystem import get_files_recursive, move_file_new_name
from jgutils import converters

from .models import Song, Album, Artist

logger = logging.getLogger(__name__)

class MusicCollection(BaseModel):

    """
    Representation of a collection of songs or audio tracks
    """

    artists: list[Artist] = []

_DEFAULT_EXTENSIONS = ["flac",
                       "mp3"]
    
def read(input_dir: Path, extensions: list[str] = None) -> MusicCollection:

    """
    This function reads a music collection from a filesystem and
    uses the metadata in each song file to construct a MusicCollection.
    """

    if not extensions:
        extensions = _DEFAULT_EXTENSIONS
     
    tags = [TingTag.get(f) for f in get_files_recursive(input_dir, extensions)]
    tags_with_titles = [t for t in tags if t.title]

    artists = []
    for tag in tags_with_titles:

        matching_artists = filter(lambda x: x.name == tag.artist, artists)
        if not matching_artists:
            artist = Artist(name=tag.artist)
            artists.append(artist)
        else:
            artist = next(matching_artists)

        if tag.album:
            album = artist.get_album(tag.album)
            if not album:
                album = Album(title=tag.album)
                artist.albums.append(album)

        song = Song(title=tag.title, identifier=str(uuid.uuid4()))
        song.formats.append(eachFile.relative_to(input_dir))
        if tag.album:
            album.songs.append(song)
        else:
            artist.songs.append(song)
        
    return MusicCollection(artists=artists)

def _refresh_collection_dirs(tags: list, collection_dir: Path): 
     for tag, song_file in tags:
        artist_dir = collection_dir / tag.artist
        if tag.album:
            move_file_new_name(song_file, artist_dir / tag.album, tag.title)
        else:
            move_file_new_name(song_file, artist_dir, tag.title)

def refresh(collection_dir: Path, work_dir: Path):

    """
    This method refreshes a music collection by making sure each song
    has a consistently named directory and that external copies are synced
    with the reference version.
    """

    ext = "flac"
    tags = [(TinyTag.get(f), f) for f in get_files_recursive(collection_dir, [ext])]
    tags_with_titles = [t for t in tags if t[0].title]

    _refresh_collection_dirs(tags_with_titles, collection_dir)

    # Get any files not existing on the sync target, convert them to a suitable format
    # and push them to the target.
    converters.convert(collection_dir, work_dir, "flac", "mp3")



       
